/*
IAX balancing proxy
Authors: Ben Smithrust, Stanislaw Pitucha

Copyright (c) 2007 Gradwell dot com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _CLIENTS_H
#define	_CLIENTS_H

typedef struct client client;

#include "connections.h"
#include "global.h"
#include "list.h"

struct client {
	struct list_ptrs list; // list of clients
	struct sockaddr_in addr;
	connection *connections[32768]; // connections by client callno
	char ip_port[32];
	int ref_count;
	int callno_count;
};

typedef struct client_search client_search;
struct client_search {
	int found;
	union {
		client *clt;
		int ip_count;
	} result;
};

struct client_search client_find_and_count(struct in_addr addr, in_port_t port);
client* client_get(struct in_addr addr, in_port_t port);
//int client_remove(in_addr_t addr, in_port_t port);
void client_destroy(client* clt);
void client_safe_destroy(client* clt);
void client_ref_up(client* clt);
void client_ref_down(client* clt);
void client_ref_down_destroy(client *clt);
void client_destroy_all(void);
int client_count(void);

int client_get_callno(client *);
void client_return_callno(client *, int);
void client_list_print(void);

extern struct list_ptrs clients[256]; // last byte of ip == hashcode

#endif	/* _CLIENTS_H */

